package controller;

import model.*;
import view.*;

import java.util.ArrayList;

public class Game {
    private int status;
    public static final int WHITE_TURN = 1, BLACK_TURN = 2, GAME_OVER = 3;

    public Game() {
        status = WHITE_TURN;
    }

    public int getStatus() {
        return status;
    }

    public boolean isWhiteTurn() {
        return status == WHITE_TURN;
    }

    public boolean isBlackTurn() {
        return status == BLACK_TURN;
    }

    public void setStatus(int status) {
        this.status = status;
    }

    public void setVariant(boolean isVariant, Custom3dModel model, Board board, BoardView boardView) {
        Position.setVariant(isVariant);
        board.setVariant(isVariant);
        boardView.refresh(board, model);
    }

    public void clickPiece(PieceView clickedPiece, Board board, BoardView boardView, CustomMaterial material, Custom3dModel model, Log log) {

        // graveyard pieces should not do anything
        if(clickedPiece.getPosition().getX() < 0) {
            return;
        }
        // if piece is colored (aka targeted), we consider it an action to apply (tile click)
        if(clickedPiece.isTargeted()) {
            TileView tile = boardView.getTile(clickedPiece.getPosition().getID());
            clickTile(tile, board, boardView, material, model, log);
            return;
        }
        if(clickedPiece.isWhite() && isWhiteTurn() || clickedPiece.isBlack() && isBlackTurn()) {
            // clicking a piece will ask her available moves
            Position position = clickedPiece.getPosition();
            board.setSelected(position);
            boardView.resetColor(material);
            boardView.setColor(material, position, CustomMaterial.SELECTED_TILE);
            ArrayList<Position> moves = board.getPiece(position).getAvailableMoves(board);

            // test if kamikaze, and shows the explosion range
            if (board.isKamikaze(position)) {
                ArrayList<Position> explosions = ((Kamikaze) board.getPiece(position)).getExplosion();
                for (Position explosion : explosions) {
                    boardView.setColor(material, explosion, CustomMaterial.EXPLOSION_TILE);
                }
                clickedPiece.setTargeted(material, CustomMaterial.EXPLOSION_TILE);
            }

            // shows all available moves on the tiles, with colors, each tile-color having a meaning
            for (Position move : moves) {

                if (board.isFree(move)) {
                    boardView.setColor(material, move, CustomMaterial.MOVABLE_TILE);
                    // test for "en passant"
                    if (board.isPawn(position) || board.isNwap(position)) {
                        Position enemy = new Position(move.getX(), position.getY());
                        boolean isEnPassant = false;
                        if (board.isEnemy(enemy, position)) {
                            if (board.isPawn(enemy)) {
                                if (((Pawn) board.getPiece(enemy)).isEnPassant()) {
                                    isEnPassant = true;
                                }
                            }
                            if (board.isNwap(enemy)) {
                                if (((Nwap) board.getPiece(enemy)).isEnPassant()) {
                                    isEnPassant = true;
                                }
                            }
                        }
                        if (isEnPassant) {
                            boardView.setColor(material, move, CustomMaterial.SPECIAL_TILE);
                            boardView.setColor(material, enemy, CustomMaterial.SPECIAL_TILE);
                            boardView.getPiece(enemy.getID()).setTargeted(material, CustomMaterial.ATTACK_TILE);
                        }
                    }
                } else if (board.isEnemy(move, position)) {
                    boardView.setColor(material, move, CustomMaterial.ATTACK_TILE);
                    boardView.getPiece(move.getID()).setTargeted(material, CustomMaterial.ATTACK_TILE);
                } else {
                    boardView.setColor(material, move, CustomMaterial.SPECIAL_TILE); // any special move
                    boardView.getPiece(move.getID()).setTargeted(material, CustomMaterial.SPECIAL_TILE);
                }
                // PROMOTION (attacking or not)
                if ((board.isPawn(position) || board.isNwap(position)) && (move.getY() == 8 || move.getY() == 1)) {
                    boardView.setColor(material, move, CustomMaterial.SPECIAL_TILE);
                }
            }
        }
    }

    public void clickTile(TileView clickedTile, Board board, BoardView boardView, CustomMaterial material, Custom3dModel model, Log log) {

        boolean moved = false; // to tell if an action/movement was used or not

        // --- Analyses the clicked tile type, and executes related actions ---
        int state = clickedTile.getState();
        Position selected = board.getSelected();
        Position arrival = clickedTile.getPosition();
        if(state != TileView.NORMAL && state != TileView.SELECTED) {
            // --- Resets enPassant ---
            for(int j = 1 ; j <= 8 ; j++) {
                for(int i = 1 ; i <= 8 ; i++) {
                    Position p = new Position(i, j);
                    if (board.isPawn(p)) {
                        ((Pawn) board.getPiece(p)).setEnPassant(false);
                    }
                    if (board.isNwap(p)) {
                        ((Nwap) board.getPiece(p)).setEnPassant(false);
                    }
                }
            }
            // --- ALL SPECIAL MOVES ---
            if(state == TileView.SPECIAL) {
                if(board.isPawn(selected) || board.isNwap(selected)) {
                    // --- PROMOTION ---
                    if(arrival.getY() == 8 || arrival.getY() == 1) {
                        // --- ATTACK (while doing promotion) ---
                        if (!board.isFree(arrival)) {
                            boardView.getPiece(arrival.getID()).resetTargeted(material); // reset color
                            boardView.killPiece(arrival); // place piece on graveyard (visual)
                        }
                        // Promotion to Queen (by default, Queen for now)
                        board.mutationOfSelectedPiece(Board.QUEEN); // (on model)
                        boardView.mutationPiece(model, selected, Custom3dModel.QUEEN); // (on view)
                        // --- MOVE (while doing promotion)  ---
                        board.movePiece(selected, arrival); // move piece (on model)
                        boardView.movePiece(selected, arrival); // move piece (on 3d view)
                        moved = true;
                    }
                    // --- PRISE EN PASSANT ---
                    else {
                        // PRE : if it's not a promotion, then it's "prise en passant", because of pawn rules
                        // we don't need to test y-position
                        Position enemy = new Position(arrival.getX(), selected.getY());
                        boardView.getPiece(enemy.getID()).resetTargeted(material); // reset color
                        boardView.killPiece(enemy); // place piece on graveyard (visual)
                        board.killPiece(enemy.getID()); // because we don't move "on it", we need to kill it manually (model)
                        Position enPassantArrival = new Position(arrival.getX(), selected.getY()+1);
                        if(board.getSelectedPiece().getColor() == Piece.BLACK) {
                            enPassantArrival.setY(selected.getY()-1);
                        }
                        board.movePiece(selected, enPassantArrival); // move piece (on model)
                        boardView.movePiece(selected, enPassantArrival); // move piece (on 3d view)
                        moved = true;
                    }
                }
                // --- CASTLING (from king) ---
                if(board.isKing(selected)) {
                    // PRE (1) : the arrival piece CAN ONLY BE ROOK, because of king rules
                    // PRE (2) : we don't need to test if rook !hasBeenMoved(), because of king rules
                    // PRE (3) : we don't need to test if king !hasBeenMoved(), because of king rules
                    int x = Board.D; // king x arrival, for west-rook
                    if(arrival.getX() == Board.H) {
                        x = Board.F; // king x arrival, for east-rook
                    }
                    Position kingArrival = new Position(x, selected.getY());
                    board.movePiece(selected, kingArrival); // move king (on model)
                    boardView.movePiece(selected, kingArrival); // move king (on 3d view)
                    board.movePiece(arrival, selected); // move rook (on model)
                    boardView.movePiece(arrival, selected); // move rook (on 3d view)
                    moved = true;
                }
                // --- CASTLING (from rook) ---
                else if(board.isRook(selected)) {
                    // PRE (1) : the arrival piece CAN ONLY BE KING, because of rook rules
                    // PRE (2) : we don't need to test if king !hasBeenMoved(), because of rook rules
                    // PRE (3) : we don't need to test if rook !hasBeenMoved(), because of rook rules
                    int x = Board.D; // king x arrival, for west-rook
                    if (selected.getX() == Board.H) {
                        x = Board.F; // king x arrival, for east-rook
                    }
                    Position kingArrival = new Position(x, arrival.getY());
                    board.movePiece(arrival, kingArrival); // move king (on model)
                    boardView.movePiece(arrival, kingArrival); // move king (on 3d view)
                    board.movePiece(selected, arrival); // move rook (on model)
                    boardView.movePiece(selected, arrival); // move rook (on 3d view)
                    moved = true;
                }
            }
            // --- EXPLOSION MOVES ---
            else if (state == TileView.EXPLOSION) {
                for(int n = 0 ; n < 64 ; n++) {
                    if(boardView.getTile(n).getState() == TileView.EXPLOSION) {
                        if(boardView.getPiece(n) != null) {
                            boardView.getPiece(n).resetTargeted(material); // reset color
                            boardView.killPiece(boardView.getPiece(n).getPosition()); // place piece on graveyard (visual)
                            board.killPiece(n);
                        }
                    }
                }
                moved = true;
            }
            // --- STANDARD MOVES ---
            else {
                // --- ATTACK ---
                if(state == TileView.ATTACK) {
                    boardView.getPiece(arrival.getID()).resetTargeted(material); // reset color
                    boardView.killPiece(arrival); // place piece on graveyard (visual)
                    // --- MAGICIAN CHANGE ---
                    if(board.isMagician(selected)) {
                        Magician magician = (Magician) board.getPiece(selected);
                        Piece enemy = board.getPiece(arrival);
                        magician.steal(enemy); // muahaha
                    }
                }
                // --- EN PASSANT activation ---
                if(Math.abs(selected.getY() - arrival.getY()) == 2) {
                    if(board.isPawn(selected)) {
                        ((Pawn) board.getPiece(selected)).setEnPassant(true);
                    }
                    if(board.isNwap(selected)) {
                        ((Nwap) board.getPiece(selected)).setEnPassant(true);
                    }
                }
                // --- MOVE ---
                board.movePiece(selected, arrival); // move piece (on model)
                boardView.movePiece(selected, arrival); // move piece (on 3d view)
                moved = true;
            }

        }
        // --- Resets board coloring, doing any action or not ---
        boardView.resetColor(material);
        // updates game status
        if(moved) {
            if(isWhiteTurn())
                setStatus(Game.BLACK_TURN);
            else if(isBlackTurn())
                setStatus(Game.WHITE_TURN);
            log.save(board, boardView);
        }
    }
}
